@@ -1,7 +1,6 @@
 
 --- Module implementing the luarocks-admin "remove" command.
 -- Removes a rock or rockspec from a rocks server.
---module("luarocks.admin_remove", package.seeall)
 local admin_remove = {}
 package.loaded["luarocks.admin_remove"] = admin_remove
 
@@ -13,6 +12,7 @@ local index = require("luarocks.index")
 local fs = require("luarocks.fs")
 local cache = require("luarocks.cache")
 
+util.add_run_function(admin_remove)
 admin_remove.help_summary = "Remove a rock or rockspec from a rocks server."
 admin_remove.help_arguments = "[--server=<server>] [--no-refresh] {<rockspec>|<rock>...}"
 admin_remove.help = [[
@@ -78,9 +78,8 @@ local function remove_files_from_server(refresh, rockfiles, server, upload_serve
    return true
 end
 
-function admin_remove.run(...)
-   local files = { util.parse_flags(...) }
-   local flags = table.remove(files, 1)
+function admin_remove.command(flags, ...)
+   local files = {...}
    if #files < 1 then
       return nil, "Argument missing. "..util.see_help("remove", "luarocks-admin")
    end
