@@ -5,21 +5,19 @@ local win32 = {}
 
 local fs = require("luarocks.fs")
 
-local cfg = require("luarocks.core.cfg")
+local cfg = require("luarocks.cfg")
 local dir = require("luarocks.dir")
-local path = require("luarocks.path")
 local util = require("luarocks.util")
 
+math.randomseed(os.time())
+
 -- Monkey patch io.popen and os.execute to make sure quoting
 -- works as expected.
 -- See http://lua-users.org/lists/lua-l/2013-11/msg00367.html
 local _prefix = "type NUL && "
 local _popen, _execute = io.popen, os.execute
-
--- luacheck: push globals io os
 io.popen = function(cmd, ...) return _popen(_prefix..cmd, ...) end
 os.execute = function(cmd, ...) return _execute(_prefix..cmd, ...) end
--- luacheck: pop
 
 --- Annotate command string for quiet execution.
 -- @param cmd string: A command-line string.
@@ -35,42 +33,23 @@ function win32.quiet_stderr(cmd)
    return cmd.." 2> NUL"
 end
 
-function win32.execute_env(env, command, ...)
-   assert(type(command) == "string")
-   local cmdstr = {}
-   for var, val in pairs(env) do
-      table.insert(cmdstr, fs.export_cmd(var, val))
-   end
-   table.insert(cmdstr, fs.quote_args(command, ...))
-   return fs.execute_string(table.concat(cmdstr, " & "))
-end
-
--- Split path into drive, root and the rest.
--- Example: "c:\\hello\\world" becomes "c:" "\\" "hello\\world"
--- if any part is missing from input, it becomes an empty string.
-local function split_root(pathname)
-   local drive = ""
+-- Split path into root and the rest.
+-- Root part consists of an optional drive letter (e.g. "C:")
+-- and an optional directory separator.
+local function split_root(path)
    local root = ""
-   local rest
 
-   local unquoted = pathname:match("^['\"](.*)['\"]$")
-   if unquoted then
-      pathname = unquoted
+   if path:match("^.:") then
+      root = path:sub(1, 2)
+      path = path:sub(3)
    end
 
-   if pathname:match("^.:") then
-      drive = pathname:sub(1, 2)
-      pathname = pathname:sub(3)
+   if path:match("^[\\/]") then
+      root = path:sub(1, 1)
+      path = path:sub(2)
    end
 
-   if pathname:match("^[\\/]") then
-      root = pathname:sub(1, 1)
-      rest = pathname:sub(2)
-   else
-      rest = pathname
-   end
-
-   return drive, root, rest
+   return root, path
 end
 
 --- Quote argument for shell processing. Fixes paths on Windows.
@@ -81,8 +60,7 @@ function win32.Q(arg)
    assert(type(arg) == "string")
    -- Use Windows-specific directory separator for paths.
    -- Paths should be converted to absolute by now.
-   local drive, root, rest = split_root(arg)
-   if root ~= "" then
+   if split_root(arg) ~= "" then
       arg = arg:gsub("/", "\\")
    end
    if arg == "\\" then
@@ -104,8 +82,7 @@ function win32.Qb(arg)
    assert(type(arg) == "string")
    -- Use Windows-specific directory separator for paths.
    -- Paths should be converted to absolute by now.
-   local drive, root, rest = split_root(arg)
-   if root ~= "" then
+   if split_root(arg) ~= "" then
       arg = arg:gsub("/", "\\")
    end
    if arg == "\\" then
@@ -129,15 +106,15 @@ function win32.absolute_name(pathname, relative_to)
    assert(type(pathname) == "string")
    assert(type(relative_to) == "string" or not relative_to)
 
-   relative_to = (relative_to or fs.current_dir()):gsub("[\\/]*$", "")
-   local drive, root, rest = split_root(pathname)
+   relative_to = relative_to or fs.current_dir()
+   local root, rest = split_root(pathname)
    if root:match("[\\/]$") then
-      -- It's an absolute path already. Ensure is not quoted.
-      return dir.normalize(drive .. root .. rest)
+      -- It's an absolute path already.
+      return pathname
    else
       -- It's a relative path, join it with base path.
       -- This drops drive letter from paths like "C:foo".
-      return dir.path(relative_to, rest)
+      return relative_to .. "/" .. rest
    end
 end
 
@@ -146,75 +123,36 @@ end
 -- @param pathname string: pathname to use.
 -- @return string: The root of the given pathname.
 function win32.root_of(pathname)
-   local drive, root, rest = split_root(fs.absolute_name(pathname))
-   return drive .. root
+   return (split_root(fs.absolute_name(pathname)))
 end
 
 --- Create a wrapper to make a script executable from the command-line.
--- @param script string: Pathname of script to be made executable.
--- @param target string: wrapper target pathname (without wrapper suffix).
+-- @param file string: Pathname of script to be made executable.
+-- @param dest string: Directory where to put the wrapper.
 -- @param name string: rock name to be used in loader context.
 -- @param version string: rock version to be used in loader context.
 -- @return boolean or (nil, string): True if succeeded, or nil and
 -- an error message.
-function win32.wrap_script(script, target, deps_mode, name, version, ...)
-   assert(type(script) == "string" or not script)
-   assert(type(target) == "string")
-   assert(type(deps_mode) == "string")
-   assert(type(name) == "string" or not name)
-   assert(type(version) == "string" or not version)
+function win32.wrap_script(file, dest, name, version)
+   assert(type(file) == "string")
+   assert(type(dest) == "string")
 
-   local wrapper = io.open(target, "wb")
+   local base = dir.base_name(file)
+   local wrapname = fs.is_dir(dest) and dest.."/"..base or dest
+   wrapname = wrapname..".bat"
+   local lpath, lcpath = cfg.package_paths()
+   lpath = util.remove_path_dupes(lpath, ";")
+   lcpath = util.remove_path_dupes(lcpath, ";")
+   local wrapper = io.open(wrapname, "w")
    if not wrapper then
-      return nil, "Could not open "..target.." for writing."
+      return nil, "Could not open "..wrapname.." for writing."
    end
-
-   local lpath, lcpath = path.package_paths(deps_mode)
-
-   local luainit = {
-      "package.path="..util.LQ(lpath..";").."..package.path",
-      "package.cpath="..util.LQ(lcpath..";").."..package.cpath",
-   }
-
-   local remove_interpreter = false
-   local base = dir.base_name(target):gsub("%..*$", "")
-   if base == "luarocks" or base == "luarocks-admin" then
-      if cfg.is_binary then
-         remove_interpreter = true
-      end
-      luainit = {
-         "package.path="..util.LQ(package.path),
-         "package.cpath="..util.LQ(package.cpath),
-      }
-   end
-
-   if name and version then
-      local addctx = "local k,l,_=pcall(require,'luarocks.loader') _=k " ..
-                     "and l.add_context('"..name.."','"..version.."')"
-      table.insert(luainit, addctx)
-   end
-
-   local argv = {
-      fs.Qb(cfg.variables["LUA"]),
-      "-e",
-      fs.Qb(table.concat(luainit, ";")),
-      script and fs.Qb(script) or "%I%",
-      ...
-   }
-   if remove_interpreter then
-      table.remove(argv, 1)
-      table.remove(argv, 1)
-      table.remove(argv, 1)
-   end
-
-   wrapper:write("@echo off\r\n")
-   wrapper:write("setlocal\r\n")
-   if not script then
-      wrapper:write([[IF "%*"=="" (set I=-i) ELSE (set I=)]] .. "\r\n")
-   end
-   wrapper:write("set "..fs.Qb("LUAROCKS_SYSCONFDIR="..cfg.sysconfdir) .. "\r\n")
-   wrapper:write(table.concat(argv, " ") .. " %*\r\n")
-   wrapper:write("exit /b %ERRORLEVEL%\r\n")
+   wrapper:write("@echo off\n")
+   local lua = dir.path(cfg.variables["LUA_BINDIR"], cfg.lua_interpreter)
+   local ppaths = "package.path="..util.LQ(lpath..";").."..package.path; package.cpath="..util.LQ(lcpath..";").."..package.cpath"
+   local addctx = "local k,l,_=pcall(require,"..util.LQ("luarocks.loader")..") _=k and l.add_context("..util.LQ(name)..","..util.LQ(version)..")"
+   wrapper:write(fs.Qb(lua)..' -e '..fs.Qb(ppaths)..' -e '..fs.Qb(addctx)..' '..fs.Qb(file)..' %*\n')
+   wrapper:write("exit /b %ERRORLEVEL%\n")
    wrapper:close()
    return true
 end
@@ -227,7 +165,7 @@ function win32.is_actual_binary(name)
    return false
 end
 
-function win32.copy_binary(filename, dest)
+function win32.copy_binary(filename, dest) 
    local ok, err = fs.copy(filename, dest)
    if not ok then
       return nil, err
@@ -237,7 +175,7 @@ function win32.copy_binary(filename, dest)
    dest = dir.dir_name(dest)
    if base:match(exe_pattern) then
       base = base:gsub(exe_pattern, ".lua")
-      local helpname = dest.."\\"..base
+      local helpname = dest.."/"..base
       local helper = io.open(helpname, "w")
       if not helper then
          return nil, "Could not open "..helpname.." for writing."
@@ -249,6 +187,14 @@ function win32.copy_binary(filename, dest)
    return true
 end
 
+function win32.chmod(filename, mode)
+   return true
+end
+
+function win32.get_permissions(filename)
+   return ""
+end
+
 --- Move a file on top of the other.
 -- The new file ceases to exist under its original name,
 -- and takes over the name of the old file.
@@ -265,41 +211,6 @@ function win32.replace_file(old_file, new_file)
    return os.rename(new_file, old_file)
 end
 
-function win32.is_dir(file)
-   file = fs.absolute_name(file)
-   file = dir.normalize(file)
-   local fd, _, code = io.open(file, "r")
-   if code == 13 then -- directories return "Permission denied"
-      fd, _, code = io.open(file .. "\\", "r")
-      if code == 2 then -- directories return 2, files return 22
-         return true
-      end
-   end
-   if fd then
-      fd:close()
-   end
-   return false
-end
-
-function win32.is_file(file)
-   file = fs.absolute_name(file)
-   file = dir.normalize(file)
-   local fd, _, code = io.open(file, "r")
-   if code == 13 then -- if "Permission denied"
-      fd, _, code = io.open(file .. "\\", "r")
-      if code == 2 then -- directories return 2, files return 22
-         return false
-      elseif code == 22 then
-         return true
-      end
-   end
-   if fd then
-      fd:close()
-      return true
-   end
-   return false
-end
-
 --- Test is file/dir is writable.
 -- Warning: testing if a file/dir is writable does not guarantee
 -- that it will remain writable and therefore it is no replacement
@@ -331,54 +242,25 @@ function win32.is_writable(file)
    return result
 end
 
+--- Create a temporary directory.
+-- @param name string: name pattern to use for avoiding conflicts
+-- when creating temporary directory.
+-- @return string or (nil, string): name of temporary directory or (nil, error message) on failure.
+function win32.make_temp_dir(name)
+   assert(type(name) == "string")
+   name = dir.normalize(name)
+
+   local temp_dir = os.getenv("TMP") .. "/luarocks_" .. name:gsub(dir.separator, "_") .. "-" .. tostring(math.floor(math.random() * 10000))
+   local ok, err = fs.make_dir(temp_dir)
+   if ok then
+      return temp_dir
+   else
+      return nil, err
+   end
+end
+
 function win32.tmpname()
-   local name = os.tmpname()
-   local tmp = os.getenv("TMP")
-   if tmp and name:sub(1, #tmp) ~= tmp then
-      name = (tmp .. "\\" .. name):gsub("\\+", "\\")
-   end
-   return name
-end
-
-function win32.current_user()
-   return os.getenv("USERNAME")
-end
-
-function win32.is_superuser()
-   return false
-end
-
-function win32.export_cmd(var, val)
-   return ("SET %s=%s"):format(var, val)
-end
-
-function win32.system_cache_dir()
-   return dir.path(fs.system_temp_dir(), "cache")
-end
-
-function win32.search_in_path(program)
-   if program:match("\\") then
-      local fd = io.open(dir.path(program), "r")
-      if fd then
-         fd:close()
-         return true, program
-      end
-
-      return false
-   end
-
-   if not program:lower():match("exe$") then
-      program = program .. ".exe"
-   end
-
-   for d in (os.getenv("PATH") or ""):gmatch("([^;]+)") do
-      local fd = io.open(dir.path(d, program), "r")
-      if fd then
-         fd:close()
-         return true, d
-      end
-   end
-   return false
+   return os.getenv("TMP")..os.tmpname()
 end
 
 return win32
