@@ -1,28 +1,27 @@
 
 local api = {}
 
-local cfg = require("luarocks.core.cfg")
+local cfg = require("luarocks.cfg")
 local fs = require("luarocks.fs")
-local dir = require("luarocks.dir")
 local util = require("luarocks.util")
 local persist = require("luarocks.persist")
 local multipart = require("luarocks.upload.multipart")
-local json = require("luarocks.vendor.dkjson")
 
 local Api = {}
 
 local function upload_config_file()
-   if not cfg.config_files.user.file then
+   local conf = cfg.which_config()
+   if not conf.user.file then
       return nil
    end
-   return (cfg.config_files.user.file:gsub("/[^/]+$", "/upload_config.lua"))
+   return (conf.user.file:gsub("/[^/]+$", "/upload_config.lua"))
 end
 
 function Api:load_config()
    local upload_conf = upload_config_file()
    if not upload_conf then return nil end
-   local config, err = persist.load_into_table(upload_conf)
-   return config
+   local cfg, err = persist.load_into_table(upload_conf)
+   return cfg
 end
 
 function Api:save_config()
@@ -37,12 +36,8 @@ function Api:save_config()
    end
    local upload_conf = upload_config_file()
    if not upload_conf then return nil end
-   local ok, err = fs.make_dir(dir.dir_name(upload_conf))
-   if not ok then
-      return nil, err
-   end
    persist.save_from_table(upload_conf, self.config)
-   fs.set_permissions(upload_conf, "read", "user")
+   fs.chmod(upload_conf, "0600")
 end
 
 function Api:check_version()
@@ -62,7 +57,7 @@ function Api:check_version()
          return nil, "Your upload client is too out of date to continue, please upgrade LuaRocks."
       end
       if res.version ~= tool_version then
-         util.warning("your LuaRocks is out of date, consider upgrading.")
+         util.printerr("Warning: Your LuaRocks is out of date, consider upgrading.")
       end
    end
    return true
@@ -109,6 +104,27 @@ local function encode_query_string(t, sep)
    return table.concat(buf)
 end
 
+-- An ode to the multitude of JSON libraries out there...
+local function require_json()
+   local list = { "cjson", "dkjson", "json" }
+   for _, lib in ipairs(list) do
+      local json_ok, json = pcall(require, lib)
+      if json_ok then
+         pcall(json.use_lpeg) -- optional feature in dkjson
+         return json_ok, json
+      end
+   end
+   local errmsg = "Failed loading "
+   for i, name in ipairs(list) do
+      if i == #list then
+         errmsg = errmsg .."and '"..name.."'. Use 'luarocks search <partial-name>' to search for a library and 'luarocks install <name>' to install one."
+      else
+         errmsg = errmsg .."'"..name.."', "
+      end
+   end
+   return nil, errmsg
+end
+
 local function redact_api_url(url)
    url = tostring(url)
    return (url:gsub(".*/api/[^/]+/[^/]+", "")) or ""
@@ -119,8 +135,10 @@ if not ltn12_ok then -- If not using LuaSocket and/or LuaSec...
 
 function Api:request(url, params, post_params)
    local vars = cfg.variables
-
-   if fs.which_tool("downloader") == "wget" then
+   local json_ok, json = require_json()
+   if not json_ok then return nil, "A JSON library is required for this command. "..json end
+   
+   if cfg.downloader == "wget" then
       local curl_ok, err = fs.is_tool_available(vars.CURL, "curl")
       if not curl_ok then
          return nil, err
@@ -134,11 +152,11 @@ function Api:request(url, params, post_params)
       url = url .. ("?" .. encode_query_string(params))
    end
    local method = "GET"
-   local out
+   local out 
    local tmpfile = fs.tmpname()
    if post_params then
       method = "POST"
-      local curl_cmd = vars.CURL.." "..vars.CURLNOCERTFLAG.." -f -L --silent --user-agent \""..cfg.user_agent.." via curl\" "
+      local curl_cmd = fs.Q(vars.CURL).." -f -k -L --silent --user-agent \""..cfg.user_agent.." via curl\" "
       for k,v in pairs(post_params) do
          local var = v
          if type(v) == "table" then
@@ -147,7 +165,7 @@ function Api:request(url, params, post_params)
          curl_cmd = curl_cmd .. "--form \""..k.."="..var.."\" "
       end
       if cfg.connection_timeout and cfg.connection_timeout > 0 then
-        curl_cmd = curl_cmd .. "--connect-timeout "..tonumber(cfg.connection_timeout).." "
+        curl_cmd = curl_cmd .. "--connect-timeout "..tonumber(cfg.connection_timeout).." " 
       end
       local ok = fs.execute_string(curl_cmd..fs.Q(url).." -o "..fs.Q(tmpfile))
       if not ok then
@@ -181,6 +199,8 @@ else -- use LuaSocket and LuaSec
 local warned_luasec = false
 
 function Api:request(url, params, post_params)
+   local json_ok, json = require_json()
+   if not json_ok then return nil, "A JSON library is required for this command. "..json end
    local server = tostring(self.config.server)
    local http_ok, http
    local via = "luasocket"
@@ -203,7 +223,7 @@ function Api:request(url, params, post_params)
    if not http_ok then
       return nil, "Failed loading socket library!"
    end
-
+   
    if not self.config.key then
       return nil, "Must have API key before performing any actions."
    end
@@ -233,32 +253,32 @@ function Api:request(url, params, post_params)
    if self.debug then
       util.printout(tostring(status))
    end
-   local pok, ret, err = pcall(json.decode, table.concat(out))
-   if pok and ret then
-      return ret
+   if status ~= 200 then
+      return nil, "API returned " .. tostring(status) .. " - " .. redact_api_url(url)
    end
-   return nil, "API returned " .. tostring(status) .. " - " .. redact_api_url(url)
+   return json.decode(table.concat(out))
 end
 
 end
 
-function api.new(args)
+function api.new(flags)
    local self = {}
    setmetatable(self, { __index = Api })
    self.config = self:load_config() or {}
-   self.config.server = args.server or self.config.server or cfg.upload.server
+   self.config.server = flags["server"] or self.config.server or cfg.upload.server
    self.config.version = self.config.version or cfg.upload.version
-   self.config.key = args.temp_key or args.api_key or self.config.key
-   self.debug = args.debug
+   self.config.key = flags["api-key"] or self.config.key
+   self.debug = flags["debug"]
    if not self.config.key then
       return nil, "You need an API key to upload rocks.\n" ..
                   "Navigate to "..self.config.server.."/settings to get a key\n" ..
                   "and then pass it through the --api-key=<key> flag."
    end
-   if args.api_key then
+   if flags["api-key"] then
       self:save_config()
    end
    return self
 end
 
 return api
+
